@@ -1,17 +1,22 @@
 //! Pure Rust implementation of the Advanced Encryption Standard
 //! (a.k.a. Rijndael)
 //!
-//! # Supported platforms
-//! This crate provides two different backends based on what target features
-//! are available:
+//! # Supported backends
+//! This crate provides multiple backends including a portable pure Rust
+//! backend as well as ones based on CPU intrinsics.
 //!
-//! - "soft" portable constant-time implementation based on [fixslicing].
-//!   Enabling the `compact` Cargo feature will reduce the code size of this
-//!   backend at the cost of decreased performance (using a modified form of
-//!   the fixslicing technique called "semi-fixslicing").
-//! - [AES-NI] accelerated implementation for `i686`/`x86_64` target
-//!   architectures with `target-feature=+aes`, as well as an accelerated
-//!   AES-CTR implementation with `target-feature=+aes,+ssse3`
+//! By default, it performs runtime detection of CPU intrinsics and uses them
+//! if they are available.
+//!
+//! ## "soft" portable backend
+//! As a baseline implementation, this crate provides a constant-time pure Rust
+//! implementation based on [fixslicing], a more advanced form of bitslicing
+//! implemented entirely in terms of bitwise arithmetic with no use of any
+//! lookup tables or data-dependent branches.
+//!
+//! Enabling the `compact` Cargo feature will reduce the code size of this
+//! backend at the cost of decreased performance (using a modified form of
+//! the fixslicing technique called "semi-fixslicing").
 //!
 //! ## ARMv8 intrinsics (nightly-only)
 //! On `aarch64` targets including `aarch64-apple-darwin` (Apple M1) and Linux
@@ -34,17 +39,20 @@
 //! Programs built in this manner will crash with an illegal instruction on
 //! CPUs which do not have AES-NI enabled.
 //!
+//! Note: runtime detection is not possible on SGX targets. Please use the
+//! afforementioned `RUSTFLAGS` to leverage AES-NI on these targets.
+//!
 //! # Usage example
 //! ```
-//! use aes::Aes128;
+//! use aes::{Aes128, Block, ParBlocks};
 //! use aes::cipher::{
 //!     BlockCipher, BlockEncrypt, BlockDecrypt, NewBlockCipher,
 //!     generic_array::GenericArray,
 //! };
 //!
 //! let key = GenericArray::from_slice(&[0u8; 16]);
-//! let mut block = GenericArray::clone_from_slice(&[0u8; 16]);
-//! let mut block8 = GenericArray::clone_from_slice(&[block; 8]);
+//! let mut block = Block::default();
+//! let mut block8 = ParBlocks::default();
 //!
 //! // Initialize cipher
 //! let cipher = Aes128::new(&key);
@@ -74,22 +82,28 @@
 //! [`block-modes`]: https://docs.rs/block-modes
 
 #![no_std]
+#![cfg_attr(
+    all(feature = "armv8", target_arch = "aarch64"),
+    feature(stdsimd, aarch64_target_feature)
+)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
-#![cfg_attr(feature = "armv8", feature(stdsimd, aarch64_target_feature))]
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg"
 )]
 #![warn(missing_docs, rust_2018_idioms)]
 
+#[cfg(all(feature = "hazmat", not(feature = "force-soft")))]
+pub mod hazmat;
+
+mod soft;
+
 use cfg_if::cfg_if;
 
 cfg_if! {
     if #[cfg(all(target_arch = "aarch64", feature = "armv8", not(feature = "force-soft")))] {
         mod armv8;
         mod autodetect;
-        mod soft;
-
         pub use autodetect::{Aes128, Aes192, Aes256};
 
         #[cfg(feature = "ctr")]
@@ -100,14 +114,11 @@ cfg_if! {
     ))] {
         mod autodetect;
         mod ni;
-        mod soft;
-
         pub use autodetect::{Aes128, Aes192, Aes256};
 
         #[cfg(feature = "ctr")]
         pub use autodetect::ctr::{Aes128Ctr, Aes192Ctr, Aes256Ctr};
     } else {
-        mod soft;
         pub use soft::{Aes128, Aes192, Aes256};
 
         #[cfg(feature = "ctr")]
@@ -122,3 +133,6 @@ pub type Block = cipher::generic_array::GenericArray<u8, cipher::consts::U16>;
 
 /// 8 x 128-bit AES blocks to be processed in parallel
 pub type ParBlocks = cipher::generic_array::GenericArray<Block, cipher::consts::U8>;
+
+/// Size of an AES block (128-bits; 16-bytes)
+pub const BLOCK_SIZE: usize = 16;
